package bankgui;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Component;
import java.awt.Font;
import java.awt.Color;
import java.awt.CardLayout;
import java.awt.Frame;
import java.awt.event.KeyEvent;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.BorderFactory;
import javax.swing.border.TitledBorder;
import javax.swing.text.MaskFormatter;
import javax.swing.JCheckBox;
import javax.swing.JFormattedTextField;
import javax.swing.border.BevelBorder;
import javax.swing.BoxLayout;

import java.text.ParseException;

import bank.ATM;
import bank.BankCard;
import bank.CentralHost;


public class ATMPanel extends JPanel {

	/**
	 * Listener to notify when ejecting the bank card is requested.
	 */
	private class EjectCardListener implements java.awt.event.ActionListener {
		public void actionPerformed(java.awt.event.ActionEvent e) {
            if (!atm.cardIsInserted())
                JOptionPane.showMessageDialog(getMainComponent(),
                        "There is no card in the ATM.",
                        "Error", JOptionPane.ERROR_MESSAGE);
            else
            {
                atm.ejectCard();
                updateATMDisplay();
                modelInteraction.cardsChanged();
            }
		}
	}


    private static final String ATM_NO_CARD_STRING = "No bank card in ATM.";
    private static final String ATM_HAS_CARD_STRING = "Card inserted.";
    private static final String ATM_AUTHENTICATED_STRING = "User authenticated.";
    private static final String ATM_NOT_AUTHENTICATED_STRING = "User not authenticated.";
    private static final Color ATM_YES_COLOUR = Color.BLUE;
    private static final Color ATM_NO_COLOUR = Color.RED;  //  @jve:decl-index=0:

	private CentralHost host;
    private ATM atm;
	private BankModelInteraction modelInteraction;
	private Frame ownerFrame;

	private static final long serialVersionUID = 1L;
	private JPanel operationsPanel = null;
	private JButton insertCardButton = null;
	private JPanel atmEmptyPanel = null;
	private JPanel atmCardPanel = null;
	private JPanel atmAuthenticatedPanel = null;
	private JButton ejectCardButton1 = null;
	private JButton ejectCardButton2 = null;
	private JButton pinEnterButton = null;
	private JFormattedTextField pinTextField = null;
	private JLabel pinLabel = null;
	private JButton balanceButton = null;
	private JButton requestStatementButton = null;
	private JButton transferButton = null;
	private JButton withdrawButton = null;
	private CardLayout operationsPanelLayout = null;  //  @jve:decl-index=0:
	private JPanel atmInfoPanel = null;
	private JLabel cardInsertedLabel = null;
	private JLabel authenticatedLabel = null;
	private JCheckBox onlineCheckBox = null;

	/**
	 * @param atm the ATM to be a GUI for
	 */
	public ATMPanel (Frame owner, ATM atm, CentralHost host, BankModelInteraction modelInteraction) {
		super();
		this.atm = atm;
		this.host = host;
		this.modelInteraction = modelInteraction;
		this.ownerFrame = owner;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		GridBagConstraints onlineCheckBoxConstraints = new GridBagConstraints();
		onlineCheckBoxConstraints.gridx = 1;
		onlineCheckBoxConstraints.anchor = GridBagConstraints.NORTHWEST;
		onlineCheckBoxConstraints.insets = new Insets(10, 5, 5, 5);
		onlineCheckBoxConstraints.gridy = 1;
		GridBagConstraints atmInfoPanelConstraints = new GridBagConstraints();
		atmInfoPanelConstraints.gridx = 1;
		atmInfoPanelConstraints.anchor = GridBagConstraints.NORTHWEST;
		atmInfoPanelConstraints.insets = new Insets(10, 5, 5, 5);
		atmInfoPanelConstraints.fill = GridBagConstraints.HORIZONTAL;
		atmInfoPanelConstraints.weightx = 1.0D;
		atmInfoPanelConstraints.gridy = 0;
		GridBagConstraints operationsPanelConstraints = new GridBagConstraints();
		operationsPanelConstraints.gridx = 0;
		operationsPanelConstraints.fill = GridBagConstraints.VERTICAL;
		operationsPanelConstraints.weightx = 0.0D;
		operationsPanelConstraints.weighty = 1.0D;
		operationsPanelConstraints.gridheight = 2;
		operationsPanelConstraints.gridy = 0;
		this.setLayout(new GridBagLayout());
		this.add(getOperationsPanel(), operationsPanelConstraints);
		this.add(getAtmInfoPanel(), atmInfoPanelConstraints);
		this.add(getOnlineCheckBox(), onlineCheckBoxConstraints);
		updateATMDisplay();
	}

	/**
	 * This method initializes operationsPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getOperationsPanel() {
		if (operationsPanel == null) {
			operationsPanel = new JPanel();
			operationsPanelLayout = new CardLayout();
			operationsPanel.setLayout(operationsPanelLayout);
			operationsPanel.setBorder(BorderFactory.createTitledBorder(null, "Operations", TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, new Font("Dialog", Font.BOLD, 12), new Color(51, 51, 51)));
			operationsPanel.add(getAtmEmptyPanel(), getAtmEmptyPanel().getName());
			operationsPanel.add(getAtmCardPanel(), getAtmCardPanel().getName());
			operationsPanel.add(getAtmAuthenticatedPanel(), getAtmAuthenticatedPanel().getName());
		}
		return operationsPanel;
	}

	/**
	 * This method initializes insertCardButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getInsertCardButton() {
		if (insertCardButton == null) {
			insertCardButton = new JButton();
			insertCardButton.setText("Insert Card");
			insertCardButton.setMnemonic(KeyEvent.VK_I);
			insertCardButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					BankCard card = modelInteraction.selectCard(getMainComponent());
					if (card == null)
						return;
                    if (atm.cardIsInserted()) {
                        JOptionPane.showMessageDialog(getMainComponent(),
                                "There is a card inserted in the ATM already.",
                                "Error", JOptionPane.ERROR_MESSAGE);
                        return;
                    }

                    atm.insertCard(card);
					modelInteraction.cardsChanged();
					updateATMDisplay();

                    if (!atm.cardIsInserted())
                        JOptionPane.showMessageDialog(getMainComponent(),
                                "The card has been confiscated.",
                                "Card confiscated", JOptionPane.INFORMATION_MESSAGE);
				}
			});
		}
		return insertCardButton;
	}

	/**
	 * This method initializes atmEmptyPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getAtmEmptyPanel() {
		if (atmEmptyPanel == null) {
			GridBagConstraints insertCardButtonConstraints = new GridBagConstraints();
			insertCardButtonConstraints.fill = GridBagConstraints.HORIZONTAL;
			insertCardButtonConstraints.gridx = 0;
			insertCardButtonConstraints.gridy = 0;
			insertCardButtonConstraints.insets = new Insets(5, 5, 5, 5);
			atmEmptyPanel = new JPanel();
			atmEmptyPanel.setLayout(new GridBagLayout());
			atmEmptyPanel.setName("nocard");
			GridBagConstraints spacerConstraints = new GridBagConstraints();
			spacerConstraints.weighty = 1.0D;
			spacerConstraints.gridx = 0;
			spacerConstraints.weightx = 1.0D;
			spacerConstraints.gridy = 1;
			atmEmptyPanel.add(getInsertCardButton(), insertCardButtonConstraints);
			atmEmptyPanel.add(new JLabel(), spacerConstraints);
		}
		return atmEmptyPanel;
	}

	/**
	 * This method initializes atmCardPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getAtmCardPanel() {
		if (atmCardPanel == null) {
			GridBagConstraints pinLabelConstraints = new GridBagConstraints();
			pinLabelConstraints.gridx = 0;
			pinLabelConstraints.gridy = 0;
			pinLabelConstraints.fill = GridBagConstraints.HORIZONTAL;
			pinLabelConstraints.insets = new Insets(5, 5, 5, 5);
			GridBagConstraints pinEnterButtonConstraints = new GridBagConstraints();
			pinEnterButtonConstraints.gridx = 2;
			pinEnterButtonConstraints.gridy = 0;
			pinEnterButtonConstraints.fill = GridBagConstraints.HORIZONTAL;
			pinEnterButtonConstraints.insets = new Insets(5, 5, 5, 5);
			pinLabel = new JLabel();
			pinLabel.setText("PIN:");
			GridBagConstraints pinTextFieldConstraints = new GridBagConstraints();
			pinTextFieldConstraints.gridx = 1;
			pinTextFieldConstraints.gridy = 0;
			pinTextFieldConstraints.fill = GridBagConstraints.HORIZONTAL;
			pinTextFieldConstraints.insets = new Insets(5, 5, 5, 5);
			GridBagConstraints ejectCardButtonConstraints = new GridBagConstraints();
			ejectCardButtonConstraints.gridx = 0;
			ejectCardButtonConstraints.gridy = 2;
			ejectCardButtonConstraints.fill = GridBagConstraints.HORIZONTAL;
			ejectCardButtonConstraints.gridwidth = 3;
			ejectCardButtonConstraints.insets = new Insets(5, 5, 5, 5);
			atmCardPanel = new JPanel();
			atmCardPanel.setLayout(new GridBagLayout());
			atmCardPanel.setName("card_notauth");
			GridBagConstraints spacerConstraints = new GridBagConstraints();
			spacerConstraints.weighty = 1.0D;
			spacerConstraints.gridx = 0;
			spacerConstraints.weightx = 1.0D;
			spacerConstraints.gridwidth = 3;
			spacerConstraints.gridy = 1;
			atmCardPanel.add(new JLabel(), spacerConstraints);
			atmCardPanel.add(getEjectCardButton1(), ejectCardButtonConstraints);
			atmCardPanel.add(getPinEnterButton(), pinEnterButtonConstraints);
			atmCardPanel.add(getPinTextField(), pinTextFieldConstraints);
			atmCardPanel.add(pinLabel, pinLabelConstraints);
		}
		return atmCardPanel;
	}

	/**
	 * This method initializes atmAuthenticatedPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getAtmAuthenticatedPanel() {
		if (atmAuthenticatedPanel == null) {
			GridBagConstraints balanceButtonConstraints = new GridBagConstraints();
			balanceButtonConstraints.gridx = 0;
			balanceButtonConstraints.gridy = 0;
			balanceButtonConstraints.fill = GridBagConstraints.HORIZONTAL;
			balanceButtonConstraints.insets = new Insets(5, 5, 5, 5);
			GridBagConstraints withdrawButtonConstraints = new GridBagConstraints();
			withdrawButtonConstraints.gridx = 0;
			withdrawButtonConstraints.gridy = 3;
			withdrawButtonConstraints.fill = GridBagConstraints.HORIZONTAL;
			withdrawButtonConstraints.insets = new Insets(5, 5, 5, 5);
			GridBagConstraints transferButtonConstraints = new GridBagConstraints();
			transferButtonConstraints.gridx = 0;
			transferButtonConstraints.gridy = 2;
			transferButtonConstraints.fill = GridBagConstraints.HORIZONTAL;
			transferButtonConstraints.insets = new Insets(5, 5, 5, 5);
			GridBagConstraints requestStatementConstraints = new GridBagConstraints();
			requestStatementConstraints.gridx = 0;
			requestStatementConstraints.gridy = 1;
			requestStatementConstraints.fill = GridBagConstraints.HORIZONTAL;
			requestStatementConstraints.insets = new Insets(5, 5, 5, 5);
			atmAuthenticatedPanel = new JPanel();
			atmAuthenticatedPanel.setLayout(new GridBagLayout());
			atmAuthenticatedPanel.setName("card_auth");
			GridBagConstraints spacerConstraints = new GridBagConstraints();
			spacerConstraints.weighty = 1.0D;
			spacerConstraints.gridx = 0;
			spacerConstraints.weightx = 1.0D;
			spacerConstraints.gridy = 4;
			GridBagConstraints ejectCardButtonConstraints = new GridBagConstraints();
			ejectCardButtonConstraints.gridx = 0;
			ejectCardButtonConstraints.gridy = 5;
			ejectCardButtonConstraints.fill = GridBagConstraints.HORIZONTAL;
			ejectCardButtonConstraints.insets = new Insets(5, 5, 5, 5);
			atmAuthenticatedPanel.add(new JLabel(), spacerConstraints);
			atmAuthenticatedPanel.add(getBalanceButton(), balanceButtonConstraints);
			atmAuthenticatedPanel.add(getRequestStatementButton(), requestStatementConstraints);
			atmAuthenticatedPanel.add(getTransferButton(), transferButtonConstraints);
			atmAuthenticatedPanel.add(getWithdrawButton(), withdrawButtonConstraints);
			atmAuthenticatedPanel.add(getEjectCardButton2(), ejectCardButtonConstraints);
		}
		return atmAuthenticatedPanel;
	}

	/**
	 * This method initializes ejectCardButton1	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getEjectCardButton1() {
		if (ejectCardButton1 == null) {
			ejectCardButton1 = new JButton();
			ejectCardButton1.setText("Eject Card");
			ejectCardButton1.setMnemonic(KeyEvent.VK_E);
			ejectCardButton1.addActionListener(new EjectCardListener());
		}
		return ejectCardButton1;
	}

	/**
	 * This method initializes ejectCardButton2	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getEjectCardButton2() {
		if (ejectCardButton2 == null) {
			ejectCardButton2 = new JButton();
			ejectCardButton2.setText("Eject Card");
			ejectCardButton2.setMnemonic(KeyEvent.VK_E);
			ejectCardButton2.addActionListener(new EjectCardListener());
		}
		return ejectCardButton2;
	}

	/**
	 * This method initializes pinEnterButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getPinEnterButton() {
		if (pinEnterButton == null) {
			pinEnterButton = new JButton();
			pinEnterButton.setText("OK");
			pinEnterButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					pinEntered(); 
				}
			});
		}
		return pinEnterButton;
	}

    /**
     * This method initializes pinTextField 
     *  
     * @return javax.swing.JTextField   
     */
    private JTextField getPinTextField() {
        if (pinTextField == null) {
            MaskFormatter maskFormatter = new MaskFormatter();
            maskFormatter.setAllowsInvalid(false);
            try {
                maskFormatter.setMask("####");
            } catch (ParseException e) {}

            pinTextField = new JFormattedTextField(maskFormatter);
            pinTextField.setColumns(4);
            pinTextField.addKeyListener(new java.awt.event.KeyAdapter() {
				public void keyPressed(java.awt.event.KeyEvent e) {
					if (e.getKeyCode() == java.awt.event.KeyEvent.VK_ENTER)
						pinEntered(); 
				}
            });
        }
        return pinTextField;
    }

	/**
	 * This method initializes balanceButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getBalanceButton() {
		if (balanceButton == null) {
			balanceButton = new JButton();
			balanceButton.setText("Check Balance");
			balanceButton.setMnemonic(KeyEvent.VK_B);
			balanceButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (!atm.isOnline())
						return;

					int balance = atm.accountBalance();

					updateATMDisplay();
					modelInteraction.cardsChanged();
					modelInteraction.accountsChanged();

					JOptionPane.showMessageDialog(getMainComponent(),
		                    "The account balance is: " + balance + ".",
		                    "Balance", JOptionPane.PLAIN_MESSAGE);
				}
			});
		}
		return balanceButton;
	}

	/**
	 * This method initializes requestStatementButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getRequestStatementButton() {
		if (requestStatementButton == null) {
			requestStatementButton = new JButton();
			requestStatementButton.setText("Request Statement");
			requestStatementButton.setMnemonic(KeyEvent.VK_S);
			requestStatementButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					atm.requestAccountStatement();

					updateATMDisplay();
					modelInteraction.cardsChanged();
					modelInteraction.accountsChanged();

					JOptionPane.showMessageDialog(getMainComponent(),
		                    "An account statement has been requested.",
		                    "Statement", JOptionPane.PLAIN_MESSAGE);
				}
			});
			try {
				requestStatementButton.setDisplayedMnemonicIndex(8);
			} catch (IllegalArgumentException exc) {}
		}
		return requestStatementButton;
	}

	/**
	 * This method initializes transferButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getTransferButton() {
		if (transferButton == null) {
			transferButton = new JButton();
			transferButton.setText("Transfer Money");
			transferButton.setMnemonic(KeyEvent.VK_T);
			transferButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (!atm.isOnline())
						return;

					int amount;
					int destinationAccountNo;
					boolean success = false;
					String errorMessage = null;

					TransferMoneyDialog transferMoneyDialog = new TransferMoneyDialog(ownerFrame);
					transferMoneyDialog.setLocationRelativeTo(getMainComponent());
					transferMoneyDialog.setVisible(true);
					if (transferMoneyDialog.cancelled())
						return;
					try {
						destinationAccountNo = transferMoneyDialog.getDestinationAccountNumber();
						amount = transferMoneyDialog.getAmount();
						if (destinationAccountNo < 0 || destinationAccountNo >= host.maxAccountNumber)
							errorMessage = "Account number must be between 0 and " + (host.maxAccountNumber - 1) + ".";
						else if (amount <= 0)
							errorMessage = "Amount must be positive.";
						else if (!host.accountExists(destinationAccountNo))
							errorMessage = "Account no. " + destinationAccountNo + " doesn't exist.";
						else if (destinationAccountNo == atm.getAccountNumber())
							errorMessage = "Transfer from an account to itself is not possible.";
						else {
							success = atm.transfer (amount, destinationAccountNo);

							updateATMDisplay();
							modelInteraction.cardsChanged();
							modelInteraction.accountsChanged();
						}
					}
					catch (IllegalArgumentException exc) {
						errorMessage = exc.getMessage();
					}

					if (errorMessage != null)
						JOptionPane.showMessageDialog(getMainComponent(),
								errorMessage, "Error", JOptionPane.ERROR_MESSAGE);
					else {
						if (success)
							JOptionPane.showMessageDialog(getMainComponent(),
									"Transfer was successful.", "Success", JOptionPane.PLAIN_MESSAGE);
						else
							JOptionPane.showMessageDialog(getMainComponent(),
									"Transfer failed.", "Failure", JOptionPane.PLAIN_MESSAGE);
					}
				}
			});
		}
		return transferButton;
	}

	/**
	 * This method initializes withdrawButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getWithdrawButton() {
		if (withdrawButton == null) {
			withdrawButton = new JButton();
			withdrawButton.setText("Withdraw");
			withdrawButton.setMnemonic(KeyEvent.VK_W);
			withdrawButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					String errorMessage = null;
					String amountString = JOptionPane.showInputDialog(getMainComponent(),
							"Amount to withdraw:", "Withdraw", JOptionPane.PLAIN_MESSAGE);
					if (amountString == null)
						return;
					try {
						int amount = Integer.parseInt(amountString);
						if (amount <= 0)
							errorMessage = "Amount must be positive.";
						else {
							int withdrawnAmount = atm.withdraw(amount);
							updateATMDisplay();
							modelInteraction.cardsChanged();
							modelInteraction.accountsChanged();
							if (withdrawnAmount == amount)
								JOptionPane.showMessageDialog(getMainComponent(),
										"" + amount + " withdrawn.",
										"Success", JOptionPane.PLAIN_MESSAGE);
							else
								JOptionPane.showMessageDialog(getMainComponent(),
										"Withdrawal of " + amount + " refused.",
										"Failure", JOptionPane.PLAIN_MESSAGE);
						}
					} catch (NumberFormatException exc) {
						errorMessage = "Invalid input (" + exc.getMessage() + ").";
					}
					if (errorMessage != null)
						JOptionPane.showMessageDialog(getMainComponent(),
								errorMessage, "Error", JOptionPane.ERROR_MESSAGE);
				}
			});
		}
		return withdrawButton;
	}

	/**
	 * This method initializes atmInfoPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getAtmInfoPanel() {
		if (atmInfoPanel == null) {
			authenticatedLabel = new JLabel();
			authenticatedLabel.setText("JLabel");
			cardInsertedLabel = new JLabel();
			cardInsertedLabel.setText("JLabel");
			atmInfoPanel = new JPanel();
			atmInfoPanel.setLayout(new BoxLayout(getAtmInfoPanel(), BoxLayout.Y_AXIS));
			atmInfoPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
			atmInfoPanel.add(cardInsertedLabel, null);
			atmInfoPanel.add(authenticatedLabel, null);
		}
		return atmInfoPanel;
	}


	private void updateATMDisplay() {
		// set cardInsertedLabel:
        if (atm.cardIsInserted()) {
            cardInsertedLabel.setText(ATM_HAS_CARD_STRING);
            cardInsertedLabel.setForeground(ATM_YES_COLOUR);
        }
        else {
            cardInsertedLabel.setText(ATM_NO_CARD_STRING);
            cardInsertedLabel.setForeground(ATM_NO_COLOUR);
        }

        // set authenticatedLabel:
        if (atm.customerIsAuthenticated()) {
            authenticatedLabel.setText(ATM_AUTHENTICATED_STRING);
            authenticatedLabel.setForeground(ATM_YES_COLOUR);
        }
        else {
            authenticatedLabel.setText(ATM_NOT_AUTHENTICATED_STRING);
            authenticatedLabel.setForeground(ATM_NO_COLOUR);
        }

		// select the appropriate operations:
        if (atm.cardIsInserted()) {
            if (atm.customerIsAuthenticated()) {
        		operationsPanelLayout.show(operationsPanel, "card_auth");
        		// transfers are possible only if the ATM is online:
        		transferButton.setEnabled(onlineCheckBox.isSelected());
        		// balance checks are possible only if the ATM is online:
        		balanceButton.setEnabled(onlineCheckBox.isSelected());
            }
            else {
        		operationsPanelLayout.show(operationsPanel, "card_notauth");
        		pinTextField.requestFocusInWindow();
            }
        }
        else
    		operationsPanelLayout.show(operationsPanel, "nocard");
	}

	/**
	 * Called, when a PIN has been entered.
	 */
	private void pinEntered() {
        int pin;
        try {
        	pin = Integer.parseInt(pinTextField.getText());
        }
        catch (NumberFormatException exc) {
            JOptionPane.showMessageDialog(this,
            		"Invalid input for PIN: \"" + pinTextField.getText() +
            		"\".\nPlease enter a four-digit number.",
                    "Error", JOptionPane.ERROR_MESSAGE);
    		pinTextField.requestFocusInWindow();
            return;
        }
        pinTextField.setValue(null);

        atm.enterPIN(pin);
        updateATMDisplay();
        modelInteraction.cardsChanged();
        if (atm.customerIsAuthenticated()) {
            JOptionPane.showMessageDialog(this,
                    "Authentication was successful.",
                    "Success", JOptionPane.PLAIN_MESSAGE);
        }
        else {
            StringBuffer msg = new StringBuffer("Authentication failed.");
            if (!atm.cardIsInserted())
                msg.append("\nCard has been confiscated.");
            JOptionPane.showMessageDialog(this, msg,
                    "Failure", JOptionPane.PLAIN_MESSAGE);
        }
	}

	/**
	 * This method initializes onlineCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getOnlineCheckBox() {
		if (onlineCheckBox == null) {
			onlineCheckBox = new JCheckBox();
			onlineCheckBox.setText("Online");
			onlineCheckBox.setSelected(atm.isOnline());
			onlineCheckBox.setMnemonic(KeyEvent.VK_O);
			onlineCheckBox.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					atm.setOnline(onlineCheckBox.isSelected());
					// various things may happen:
					updateATMDisplay();
					modelInteraction.accountsChanged();
					modelInteraction.cardsChanged();
				}
			});
		}
		return onlineCheckBox;
	}

	private Component getMainComponent() {
		return this;
	}
}  //  @jve:decl-index=0:visual-constraint="10,10"
