package bankgui;

import java.awt.GridBagLayout;

import javax.swing.JPanel;
import javax.swing.JLabel;

import bank.Account;
import bank.Transaction;

import java.awt.GridBagConstraints;
import javax.swing.JTextArea;
import java.awt.Insets;
import javax.swing.JScrollPane;

public class AccountInfoPanel extends JPanel {

	private Account account;
	private BankModelInteraction modelInteraction;

	private static final long serialVersionUID = 1L;

	private JTextArea transactionsTextArea = null;
	private JLabel accountNumberLabel = null;
	private JLabel balanceLabel = null;
	private JLabel transactionsLabel = null;
	private JScrollPane transactionsScrollPane = null;

	/**
	 * @param account account to display information for
	 */
	public AccountInfoPanel(Account account, BankModelInteraction modelInteraction) {
		super();
		this.account = account;
		this.modelInteraction = modelInteraction;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		GridBagConstraints transactionsScrollPaneConstraints = new GridBagConstraints();
		transactionsScrollPaneConstraints.fill = GridBagConstraints.BOTH;
		transactionsScrollPaneConstraints.gridy = 3;
		transactionsScrollPaneConstraints.weightx = 1.0;
		transactionsScrollPaneConstraints.weighty = 1.0;
		transactionsScrollPaneConstraints.insets = new Insets(0, 20, 0, 20);
		transactionsScrollPaneConstraints.gridx = 0;
		GridBagConstraints transactionsLabelConstraints = new GridBagConstraints();
		transactionsLabelConstraints.gridx = 0;
		transactionsLabelConstraints.insets = new Insets(0, 0, 5, 0);
		transactionsLabelConstraints.anchor = GridBagConstraints.WEST;
		transactionsLabelConstraints.gridy = 2;
		transactionsLabel = new JLabel();
		GridBagConstraints balanceLabelConstraints = new GridBagConstraints();
		balanceLabelConstraints.gridx = 0;
		balanceLabelConstraints.anchor = GridBagConstraints.WEST;
		balanceLabelConstraints.insets = new Insets(0, 0, 5, 0);
		balanceLabelConstraints.gridy = 1;
		balanceLabel = new JLabel();
		GridBagConstraints accountNumberLabelConstraints = new GridBagConstraints();
		accountNumberLabelConstraints.anchor = GridBagConstraints.WEST;
		accountNumberLabelConstraints.insets = new Insets(0, 0, 5, 0);
		accountNumberLabel = new JLabel();
		this.setSize(300, 200);
		this.setLayout(new GridBagLayout());
		this.add(accountNumberLabel, accountNumberLabelConstraints);
		this.add(balanceLabel, balanceLabelConstraints);
		this.add(transactionsLabel, transactionsLabelConstraints);
		this.add(getTransactionsScrollPane(), transactionsScrollPaneConstraints);
		showAccountInfo();
	}

	/**
	 * This method initializes transactionsTextArea	
	 * 	
	 * @return javax.swing.JTextArea	
	 */
	private JTextArea getTransactionsTextArea() {
		if (transactionsTextArea == null) {
			transactionsTextArea = new JTextArea();
			transactionsTextArea.setEditable(false);
			transactionsTextArea.setRows(10);
		}
		return transactionsTextArea;
	}

	/**
	 * This method initializes transactionsScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getTransactionsScrollPane() {
		if (transactionsScrollPane == null) {
			transactionsScrollPane = new JScrollPane();
			transactionsScrollPane.setViewportView(getTransactionsTextArea());
		}
		return transactionsScrollPane;
	}

	private void showAccountInfo() {
        StringBuffer transactionsString = new StringBuffer();
        Transaction transactions[] = account.getTransactions();
    	int transactionCount = account.getTransactionCount();

        for (int i = 0; i < transactionCount; i++) {
           	transactionsString.append(" " + transactions[i].toString() + "  (" +
           			modelInteraction.printDate(transactions[i].getDate()) + ") \n");
        }
        // delete last newline character, if present:
        if (transactionsString.length() > 0)
        	transactionsString.deleteCharAt(transactionsString.length() - 1);

		accountNumberLabel.setText("Account number: " + account.getAccountNumber());
		balanceLabel.setText("Balance: " + account.accountBalance());
		transactionsLabel.setText("Transactions:");
        transactionsTextArea.setText(transactionsString.toString());
	}
}
