package bankgui;

import javax.swing.JPanel;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JFormattedTextField;
import javax.swing.JButton;
import javax.swing.text.MaskFormatter;

import java.awt.Frame;
import java.awt.Window;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.FlowLayout;

import java.text.ParseException;


public class CreateCardDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private JPanel mainPane = null;
	private JLabel accountNoLabel = null;
	private JFormattedTextField pinTextField = null;
	private JLabel pinLabel = null;
	private JTextField accountNoTextField = null;
	private JButton okButton = null;
	private boolean cancelled = false;
	private JButton cancelButton = null;
	private JPanel buttonPanel = null;

	/**
	 * @param owner
	 */
	public CreateCardDialog(Frame owner) {
		super(owner, "Create card", true);
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(221, 126);
		this.setContentPane(getMainPane());
		this.addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent e) {
				cancelled = true;
			}
		});
	}

	/**
	 * This method initializes mainPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getMainPane() {
		if (mainPane == null) {
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridwidth = 2;
			gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints.gridy = 3;
			GridBagConstraints accountNoTextFieldConstraints = new GridBagConstraints();
			accountNoTextFieldConstraints.fill = GridBagConstraints.VERTICAL;
			accountNoTextFieldConstraints.gridy = 0;
			accountNoTextFieldConstraints.weightx = 1.0;
			accountNoTextFieldConstraints.anchor = GridBagConstraints.WEST;
			accountNoTextFieldConstraints.insets = new Insets(0, 5, 0, 0);
			accountNoTextFieldConstraints.gridx = 1;
			GridBagConstraints pinLabelConstraints = new GridBagConstraints();
			pinLabelConstraints.gridx = 0;
			pinLabelConstraints.anchor = GridBagConstraints.EAST;
			pinLabelConstraints.insets = new Insets(0, 10, 0, 0);
			pinLabelConstraints.gridy = 2;
			pinLabel = new JLabel();
			pinLabel.setText("PIN:");
			GridBagConstraints pinTextFieldConstraints = new GridBagConstraints();
			pinTextFieldConstraints.fill = GridBagConstraints.VERTICAL;
			pinTextFieldConstraints.gridy = 2;
			pinTextFieldConstraints.weightx = 1.0;
			pinTextFieldConstraints.anchor = GridBagConstraints.WEST;
			pinTextFieldConstraints.insets = new Insets(0, 5, 0, 0);
			pinTextFieldConstraints.gridx = 1;
			GridBagConstraints accountNoLabelConstraints = new GridBagConstraints();
			accountNoLabelConstraints.gridx = 0;
			accountNoLabelConstraints.insets = new Insets(0, 10, 0, 0);
			accountNoLabelConstraints.gridy = 0;
			accountNoLabel = new JLabel();
			accountNoLabel.setText("Account no.:");
			mainPane = new JPanel();
			mainPane.setLayout(new GridBagLayout());
			mainPane.add(accountNoLabel, accountNoLabelConstraints);
			mainPane.add(getAccountNoTextField(), accountNoTextFieldConstraints);
			mainPane.add(pinLabel, pinLabelConstraints);
			mainPane.add(getPinTextField(), pinTextFieldConstraints);
			mainPane.add(getButtonPanel(), gridBagConstraints);
		}
		return mainPane;
	}

	/**
	 * This method initializes pinTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getPinTextField() {
        if (pinTextField == null) {
            MaskFormatter formatter = new MaskFormatter();
            formatter.setAllowsInvalid(false);
            try {
                formatter.setMask("####");
            } catch (ParseException e) {}

            pinTextField = new JFormattedTextField(formatter);
            pinTextField.setColumns(4);
            pinTextField.addKeyListener(new java.awt.event.KeyAdapter() {   
				public void keyPressed(java.awt.event.KeyEvent e) {
					switch (e.getKeyCode()) {
						case java.awt.event.KeyEvent.VK_ESCAPE:
							cancelled = true;
							getMainWindow().dispose();
							break;
						case java.awt.event.KeyEvent.VK_ENTER:
							getMainWindow().dispose();
							break;
					}
				}
			});
        }
        return pinTextField;
	}

	/**
	 * This method initializes accountNoTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getAccountNoTextField() {
		if (accountNoTextField == null) {
            accountNoTextField = new JTextField();
			accountNoTextField.setColumns(6);
			accountNoTextField.addKeyListener(new java.awt.event.KeyAdapter() {   
				public void keyPressed(java.awt.event.KeyEvent e) {
					switch (e.getKeyCode()) {
						case java.awt.event.KeyEvent.VK_ESCAPE:
							cancelled = true;
							getMainWindow().dispose();
							break;
						case java.awt.event.KeyEvent.VK_ENTER:
							pinTextField.requestFocusInWindow();
							break;
					}
				}
			});
		}
		return accountNoTextField;
	}

	/**
	 * This method initializes okButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText("OK");
			okButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					getMainWindow().dispose();
				}
			});
		}
		return okButton;
	}

	/**
	 * @return the account number for the card to create
	 * @throws IllegalArgumentException if the user input has been invalid
	 */
	public int getAccountNumber() throws IllegalArgumentException {
		int accountNumber;
		try {
			accountNumber = Integer.parseInt(accountNoTextField.getText());
		}
		catch (NumberFormatException exc) {
			throw new IllegalArgumentException("Invalid input for account number: \"" + accountNoTextField.getText() + "\"");
		}
		return accountNumber;
	}

	/**
	 * @return the PIN for the card to create
	 * @throws IllegalArgumentException if the user input has been invalid
	 */
	public int getPIN() throws IllegalArgumentException {
		int pin;
		try {
			pin = Integer.parseInt(pinTextField.getText());
		}
		catch (NumberFormatException exc) {
			throw new IllegalArgumentException("Invalid input for PIN: \"" + pinTextField.getText() + "\"");
		}
		return pin;
	}

	/**
	 * @return <code>true</code> iff the user cancelled card creation
	 */
	public boolean cancelled() {
		return cancelled;
	}

	/**
	 * @return the main window of the application
	 */
	private Window getMainWindow()
	{
		return this;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText("Cancel");
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					cancelled = true;
					getMainWindow().dispose();
				}
			});
		}
		return cancelButton;
	}

	/**
	 * This method initializes buttonPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getButtonPanel() {
		if (buttonPanel == null) {
			buttonPanel = new JPanel();
			buttonPanel.setLayout(new FlowLayout());
			buttonPanel.add(getOkButton(), null);
			buttonPanel.add(getCancelButton(), null);
		}
		return buttonPanel;
	}
}
